'use strict';

// eslint-disable-next-line no-underscore-dangle
function babelPresetMoon(api, {
  decorators,
  loose,
  modules,
  react,
  solid,
  targets
} = {}) {
  let looseMode = loose ?? false;
  const plugins = ['@babel/plugin-proposal-export-default-from', '@babel/plugin-transform-export-namespace-from', 'babel-plugin-conditional-invariant', 'babel-plugin-env-constants'];

  // When using decorators, we must apply loose to explicit plugins
  // https://babeljs.io/docs/en/babel-plugin-proposal-decorators#legacy
  if (decorators) {
    looseMode = true;
    plugins.unshift(['@babel/plugin-proposal-decorators', {
      legacy: true
    }], ['@babel/plugin-transform-class-properties', {
      loose: true
    }], ['@babel/plugin-transform-private-methods', {
      loose: true
    }]);
  }
  const presets = [['@babel/preset-env', {
    bugfixes: true,
    exclude: ['@babel/plugin-transform-regenerator', '@babel/plugin-transform-async-to-generator'],
    loose: looseMode,
    modules: modules ? false : 'auto',
    shippedProposals: true,
    // Only target node since this is for development
    // Revisit in Babel v8: https://babeljs.io/docs/en/options#no-targets
    targets: targets ?? {
      node: 'current'
    },
    useBuiltIns: false
  }], ['@babel/preset-typescript', {
    allowDeclareFields: true
  }]];
  if (react) {
    presets.push(['@babel/preset-react', {
      development: process.env.NODE_ENV !== "production",
      runtime: react === 'automatic' ? 'automatic' : 'classic'
    }]);
  }
  if (solid) {
    presets.push(['babel-preset-solid', {
      generate: solid === 'ssr' ? 'ssr' : 'dom',
      hydratable: true
    }]);
  }
  return {
    plugins,
    presets
  };
}
module.exports = babelPresetMoon;
//# sourceMappingURL=index.js.map
